<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class Clean
 * @package FiloBlu\MappConnector\Console\Command
 */
class Clean extends Command
{

    /**
     * @var \FiloBlu\MappConnector\Model\Jobs
     */
    protected $_jobs;

    /**
     * @var \FiloBlu\MappConnector\Model\Mapp
     */
    protected $_mapp;

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_loggerHelper;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * Clean constructor.
     * @param \FiloBlu\MappConnector\Model\Mapp $mapp
     * @param \FiloBlu\MappConnector\Model\Jobs $jobs
     * @param \Magento\Framework\App\State $state
     * @param \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Mapp $mapp,
        \FiloBlu\MappConnector\Model\Jobs $jobs,
        \Magento\Framework\App\State $state,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper,
        \FiloBlu\MappConnector\Model\Config $config
    )
    {
        $this->_jobs = $jobs;
        $this->_mapp = $mapp;
        $this->_state = $state;
        $this->_loggerHelper = $loggerHelper;
        $this->_config = $config;
        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    )
    {
        //Area Code Fix
        try {
            $this->_state->setAreaCode(\Magento\Framework\App\Area::AREA_GLOBAL);
        }
        catch (\Exception $e) {
            //Area code is already set
        }

        //Clean records by configuration properties
        $result = $this->_jobs->cleanRecords($this->_config);
        $this->_loggerHelper->info($result);

    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("mapp:clean");
        $this->setDescription("Clean old processed jobs (--DISABLED--)");
        parent::configure();
    }
}
