<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;

class Config {

    /**
     * XML PATHS
     */
    const MAPPCONNECTOR_API_BASE_URL = "mappconnector/connection/api_base_url";
    const MAPPCONNECTOR_API_SYSTEM_NAME = "mappconnector/connection/api_system_name";
    const MAPPCONNECTOR_API_USERNAME = "mappconnector/connection/api_username";
    const MAPPCONNECTOR_API_PASSWORD = "mappconnector/connection/api_password";

    const MAPPCONNECTOR_FTP_URL = "mappconnector/export/ftp_url";
    const MAPPCONNECTOR_FTP_LOGIN = "mappconnector/export/ftp_login";
    const MAPPCONNECTOR_FTP_PASSWORD = "mappconnector/export/ftp_password";
    const MAPPCONNECTOR_FTP_ROOT_DIR = "mappconnector/export/ftp_root_dir";
    const MAPPCONNECTOR_LOCAL_ROOT_DIR = "mappconnector/export/local_root_dir";

    /**
     * @var ScopeConfigInterface
     */
    private $_config;

    /**
     * @var
     */
    private $_entity;

    /**
     * Config constructor.
     * @param ScopeConfigInterface $config
     */
    public function __construct(ScopeConfigInterface $config)
    {
        $this->_config = $config;
    }

    /**
     * @return mixed
     */
    public function getApiBaseUrl()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_API_BASE_URL);
    }

    /**
     * @return string
     */
    public function getApiPrefix()
    {
        return "/".$this->_config->getValue(self::MAPPCONNECTOR_API_SYSTEM_NAME)."/api/rest/v6";
    }

    /**
     * @return mixed
     */
    public function getApiUsername()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_API_USERNAME);
    }

    /**
     * @return mixed
     */
    public function getApiPassword()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_API_PASSWORD);
    }

    /**
     * @return mixed
     */
    public function getFTPUrl()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_FTP_URL);
    }

    /**
     * @return string
     */
    public function getFTPLogin()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_FTP_LOGIN);
    }

    /**
     * @return mixed
     */
    public function getFTPPassword()
    {
        return $this->_config->getValue(self::MAPPCONNECTOR_FTP_PASSWORD);
    }

    /**
     * @return mixed
     */
    public function getFTPRootDir()
    {
        $path = str_replace('\\','/',$this->_config->getValue(self::MAPPCONNECTOR_FTP_ROOT_DIR));
        return str_replace('/',DIRECTORY_SEPARATOR,trim($path,'/'));
    }

    /**
     * @return mixed
     */
    public function getLocalRootDir()
    {
        $path = str_replace('\\','/',$this->_config->getValue(self::MAPPCONNECTOR_LOCAL_ROOT_DIR));
        return str_replace('/',DIRECTORY_SEPARATOR,trim($path,'/'));
    }

    /**
     * @return array
     */
    public function getDomainsToExclude()
    {
        return array_map('trim',explode(',',$this->getConfigValue('mappconnector/general/exclude_mail')));
    }

    /**
     * @param $entity
     * @return string
     */
    public function getEntityXmlPath(){
        $en = explode('_',$this->_entity);
        return 'mappconnector/'.$en[0].'_mapping/'.$this->_entity;
    }

    /**
     * @param $config_path
     * @return mixed
     */
    public function getConfigValue($config_path,$unserialize = false)
    {
        return (!$unserialize)?$this->_config->getValue($config_path):unserialize($this->_config->getValue($config_path));
    }

    /**
     * @param $entity
     */
    public function setEntity($entity)
    {
        $this->_entity = $entity;
    }

    /**
     * @param $entity
     * @return array
     */
    public function getNormalizedConfigObject(){
        $config = [];
        $c = $this->getConfigValue($this->getEntityXmlPath($this->_entity),true);
        foreach($c as $v){
            $config[$v[$this->_entity]] = $v[$this->_entity.'_mapp_field'];
        }
        return $config;
    }

    /**
     * @param $entity
     * @return array
     */
    public function getNormalizedConfigObjectByPath($path,$entity){
        $config = [];
        $c = $this->getConfigValue($path,true);
        foreach($c as $v){
            $config[$v[$entity]] = $v[$entity.'_mapp_field'];
        }
        return $config;
    }

    /**
     * @param $map
     * @param $array
     * @return array
     */
    public function getMappedArray($map,$array){
        $result = [];
        foreach ($array as $k => $v) {
            if (in_array($k, array_keys($map))) {
                $result[$map[$k]] = $v;
            }
        }
        return $result;
    }

    /**
     * @param $adapter
     * @return mixed
     */
    public function getAdapter($adapter)
    {
        $adapterObject = [
            'rest' => '\FiloBlu\MappConnector\Adapter\RestAdapter',
            'ftp' => '\FiloBlu\MappConnector\Adapter\FtpAdapter'
        ];
        return $adapterObject[$adapter];
    }

}