<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

/**
 * Class QuoteItem
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class QuoteItem extends \FiloBlu\MappConnector\Model\From\Entity\AbstractEntity implements \FiloBlu\MappConnector\Model\From\Entity\EntityInterface  {

    //TODO: Get only abandoned cart

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'quote_item';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = '\Magento\Quote\Model\Quote\Item';

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'quote_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'quote_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = '\FiloBlu\MappConnector\Model\To\Mapp\FTP';

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'quote_id', 'primaryMapp' => 'email'];

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/quote_mapping/quote_item';

    /**
     * @var
     */
    protected $_quote;

    /**
     * QuoteItem constructor.
     * @param Quote $quote
     * @param \FiloBlu\MappConnector\Model\Config $config
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\From\Entity\Quote $quote,
        \FiloBlu\MappConnector\Model\Config $config
    )
    {
        $this->_config = $config;
        $this->_quote = $quote;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getDefaultData($record){
        $result = [];

        $result['key'] = 'quote-'.$record[$this->_entityMappPrimary['primaryDB']].'-item-'.$record['item_id'];

        return $result;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary,$id)
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('quote_item'); //gives table name with prefix

        //Select Data from table
        $sql = "Select * FROM " . $tableName.' WHERE '.$primary.' = '.$id;
        $result = $connection->fetchAll($sql);

        return $result;
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record){
        $result = [];

        //Prendo le informazioni dai moduli correlati
        $result = array_merge(
            $result,
            $this->getRelatedModelData($this->_quote,$record['quote_id'])
        );

        return $result;
    }

    /**
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
        $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $connection = $resource->getConnection();
        $tableName = $resource->getTableName('quote'); //gives table name with prefix
        $tableNameJobs = $resource->getTableName('filoblu_mappconnector_jobs');

        if($type == 'full') {
            //Full
            //$sql = "Select * FROM " . $tableName;
            $sql = "SELECT count(*) 
                    FROM 
                    $tableName
                    WHERE 
                    is_active = 1 
                    AND 
                    items_count > 0 
                    AND 
                    customer_id IS NOT NULL 
                    AND 
                    TIMESTAMPDIFF(SECOND,created_at,NOW()) > ".$this->_config->getConfigValue('mappconnector/quote_mapping/abandoned_cart_time');

            $result = $connection->fetchAll($sql);
            return $result;
        }
        else {
            //Incremental
            $sql = "SELECT 
                        ce.entity_id as entity_id
                    FROM
                        ".$tableName." AS ce
                    LEFT JOIN
                        ".$tableNameJobs." AS mj 
                    ON 
                        mj.entity_id = ce.entity_id
                    AND 
                        mj.entity = 'quote'
                    WHERE 
                        mj.status is null
                    AND
                        ce.is_active = 1 
                    AND 
                        ce.items_count > 0 
                    AND 
                        ce.customer_id IS NOT NULL 
                    AND 
                        TIMESTAMPDIFF(SECOND,ce.created_at,NOW()) > ".$this->_config->getConfigValue('mappconnector/quote_mapping/abandoned_cart_time');

            return $connection->fetchAll($sql);
        }
    }

    /**
     * @return string
     */
    public function getNormalizedEntityModelPrimary() {
        return $this->_quote->getEntityModelPrimary();
    }

}